@@ -27,14 +27,13 @@ class Association(object):
 
     An example showing the difference between source and target classes::
 
-        class Car(pyperry.Base): pass
-        class Part(pyperry.Base): pass
+        class Car(pyperry.Base):
+            # Car is the target class and Part is the source class
+            parts = HasMany(class_name='Part')
 
-        # Car is the target class and Part is the source class
-        Car.has_many('parts', class_name='Part')
-
-        # Part is the target class and Car is the source class
-        Part.belongs_to('car', class_name='Car')
+        class Part(pyperry.Base):
+            # Part is the target class and Car is the source class
+            car = BelongsTo(class_name='Car')
 
         c = Car.first()
         # returns a collection of Part models (Part is the source class)
@@ -80,9 +79,9 @@ class Association(object):
 
     """
 
-    def __init__(self, target_klass, id, **kwargs):
-        self.target_klass = target_klass
-        self.id = id
+    def __init__(self, **kwargs):
+        self.target_klass = kwargs.get('target_klass')
+        self.id = kwargs.get('id')
         self.options = kwargs
 
     def __call__(self, obj):
@@ -90,6 +89,33 @@ class Association(object):
             return self.scope(obj)
         return self.scope(obj).first()
 
+    def __get__(self, instance, owner):
+        """get the relation/instance for this association"""
+        if instance is None:
+            return self
+        elif hasattr(instance, self.cache_id):
+            return getattr(instance, self.cache_id)
+        else:
+            val = None
+            if(self.collection()):
+                val = self.scope(instance)
+            else:
+                val = self.scope(instance).first()
+            setattr(instance, self.cache_id, val)
+            return val
+
+    def __set__(self, instance, value):
+        """sets the given value to the cache"""
+        setattr(instance, self.cache_id, value)
+
+    def __delete__(self, instance):
+        """clears the cache attribute"""
+        delattr(instance, self.cache_id)
+
+    @property
+    def cache_id(self):
+        return '_%s_cache' % self.id
+
     def type(self):
         raise NotImplementedError, 'You must define the type in subclasses.'
 
@@ -208,6 +234,14 @@ class BelongsTo(Association):
 
     """
 
+    def __set__(self, instance, value):
+        super(BelongsTo, self).__set__(instance, value)
+
+        setattr(instance, self.get_foreign_key(), value.pk_value())
+
+        if self.polymorphic:
+            setattr(instance, self.polymorphic_type(), value.__class__.__name__)
+
     def type(self):
         return 'belongs_to'
 
@@ -290,18 +324,17 @@ class Has(Association):
         Builds conditions on top of the base_scope generated from any finder
         options set with the association::
 
-            has_many('widgets', klass=Widget, foreign_key='widget_id')
-            has_many('comments', as_='parent')
+            widgets = HasMany(klass=Widget, foreign_key='widget_id')
+            comments = HasMany(as_='parent')
 
         In addition to any finder options included with the association options
         the following will be added::
 
-            where('widget_id = %s ' % target['id'])
+            where(widget_id=target['id'])
 
         Or for the polymorphic :comments association::
 
-            where('parent_id = %s AND parent_type = %s' % (target['id'],
-            target.class))
+            where(parent_id=target['id'], parent_type=target.class)
 
         """
         pk_attr = self.primary_key()
@@ -322,13 +355,22 @@ class Has(Association):
                 })
             return scope
 
-class HasMany(Has):
+def HasMany(**kwargs):
     """
-    The C{HasMany} class simply declares that the L{Has} association is a
-    collection of type C{'has_many'}.
+    Wrapper constructor to detect through relationships and initialize the
+    correct class
+    """
+    if kwargs.get('through') is not None:
+        return HasManyThrough(**kwargs)
+    else:
+        return HasManyDirectly(**kwargs)
+
+class HasManyDirectly(Has):
+    """
+    The C{HasManyDirectly} class simply declares that the L{Has} association is
+    a collection of type C{'has_many'}.
 
     """
-
     def type(self):
         return 'has_many'
 
@@ -341,7 +383,6 @@ class HasOne(Has):
     not a collection and has a type of C{'has_one'}.
 
     """
-
     def type(self):
         return 'has_one'
 
@@ -388,24 +429,24 @@ class HasManyThrough(Has):
     C{Device}::
 
         class Internet(pyperry.Base):
-            def _config(cls):
-                cls.attributes('id')
-                cls.has_many('connected_devices', through='networks', source='devices')
-                cls.has_many('networks', class_name='Network')
+            id = Field()
+
+            connected_devices = HasMany(through'networks', source='devices')
+            networks = HasMany(class_name='Network')
 
         class Network(pyperry.Base):
-            def _config(cls):
-                cls.attributes('id', 'internet_id')
-                cls.belongs_to('internet', class_name='Internet')
-                cls.has_many('devices', class_name='Device')
+            id = Field()
+            internet_id = Field()
+
+            internet = BelongsTo(class_name='Internet')
+            devices = HasMany(class_name='Device')
 
         class Device(pyperry.Base):
-            def _config(cls):
-                cls.attributes('id', 'network_id')
-                cls.belongs_to('network', class_name='Network')
+            id = Field()
+            network_id = Field()
 
+            network = BelongsTo(class_name='Network')
     """
-
     def type(self):
         return 'has_many_through'
 
@@ -466,3 +507,4 @@ class HasManyThrough(Has):
                 relation = relation.where({ poly_type_attr: poly_type_name })
 
         return relation
+
